#define F_CPU 8000000UL  // vor #include <util/delay.h>

#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

// I2C Kommunikation, SSD1306 Controller chip, 0,96 Zoll Display von AZ-Delivery
// SCL PB0
// SDA PB1

#define HALF_CLK 1
#define FULL_CLK 2

#define SCL_HIGH       (PORTB |=  (1<<0)) 
#define SCL_LOW        (PORTB &= ~(1<<0))
#define SCL_SET_WRITE  (DDRB  |=  (1<<0))

#define SDA_HIGH       (PORTB |=  (1<<1)) 
#define SDA_LOW        (PORTB &= ~(1<<1))
#define SDA_SET_READ   (DDRB  &= ~(1<<1))
#define SDA_SET_WRITE  (DDRB  |=  (1<<1))
#define SDA_READ       (PINB  &   (1<<1))

// SLA (0x3C) + WRITE_MODE (0x00) =  0x78 (0b01111000)
#define OLED_I2C_ADDRESS   0x3C

// Control byte
#define OLED_CONTROL_BYTE_CMD_SINGLE    0x80
#define OLED_CONTROL_BYTE_CMD_STREAM    0x00
#define OLED_CONTROL_BYTE_DATA_STREAM   0x40

// Fundamental commands (pg.28)
#define OLED_CMD_SET_CONTRAST           0x81    // follow with 0x7F
#define OLED_CMD_DISPLAY_RAM            0xA4
#define OLED_CMD_DISPLAY_ALLON          0xA5
#define OLED_CMD_DISPLAY_NORMAL         0xA6
#define OLED_CMD_DISPLAY_INVERTED       0xA7
#define OLED_CMD_DISPLAY_OFF            0xAE
#define OLED_CMD_DISPLAY_ON             0xAF

// Addressing Command Table (pg.30)
#define OLED_CMD_SET_MEMORY_ADDR_MODE   0x20    // follow with 0x00 = HORZ mode = Behave like a KS108 graphic LCD
#define OLED_CMD_SET_COLUMN_RANGE       0x21    // can be used only in HORZ/VERT mode - follow with 0x00 and 0x7F = COL127
#define OLED_CMD_SET_PAGE_RANGE         0x22    // can be used only in HORZ/VERT mode - follow with 0x00 and 0x07 = PAGE7

// Hardware Config (pg.31)
#define OLED_CMD_SET_DISPLAY_START_LINE     0x40
#define OLED_CMD_SET_SEGMENT_MODE_NORMAL    0xA0
#define OLED_CMD_SET_SEGMENT_MODE_REMAP     0xA1    
#define OLED_CMD_SET_MUX_RATIO              0xA8    // follow with 0x3F = 64 MUX
#define OLED_CMD_SET_COM_SCAN_MODE_NORMAL   0xC0 
#define OLED_CMD_SET_COM_SCAN_MODE_REMAPPED 0xC8   
#define OLED_CMD_SET_DISPLAY_OFFSET         0xD3    // follow with 0x00
#define OLED_CMD_SET_COM_PIN_MAP            0xDA    // follow with 0x12
#define OLED_CMD_NOP                        0xE3    // NOP

// Timing and Driving Scheme (pg.32)
#define OLED_CMD_SET_DISPLAY_CLK_DIV    0xD5    // follow with 0x80
#define OLED_CMD_SET_PRECHARGE          0xD9    // follow with 0xF1
#define OLED_CMD_SET_VCOMH_DESELCT      0xDB    // follow with 0x30

// Charge Pump (pg.62)
#define OLED_CMD_SET_CHARGE_PUMP        0x8D    // follow with 0x14

// LCDAssistant.exe mit Standardeinstellungen
const uint8_t font[] PROGMEM = {
0x00, 0x70, 0x70, 0x38, 0x38, 0x1C, 0x1E, 0x0E, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x38, 0x3C, 0x1E, 0x0E, 0x0E, 0x07, 0x07,
0x07, 0x07, 0x07, 0x0F, 0x0F, 0x3E, 0xFE, 0xFC, 0xF8, 0xE0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x38, 0x3C, 0x1E, 0x0E, 0x0E, 0x07, 0x07, 0x07, 0x07, 0x07, 0x0F, 0x0F, 0x1E, 0xFE, 0xFC, 0xF8,
0xF0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xC0, 0xE0, 0xF8,
0x7E, 0x3F, 0x0F, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xC0, 0xF8, 0xFF, 0xFF,
0x3F, 0x0F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xE0, 0xE0, 0xE0, 0xE0, 0xE0, 0xE0,
0xE0, 0xF0, 0xF0, 0xF8, 0xFC, 0x9F, 0x9F, 0x0F, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0xC0, 0xE0, 0xF8, 0x7E, 0x3F, 0x0F, 0x07, 0x01, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x7F, 0x7F, 0x7F, 0x7F, 0x70, 0x70, 0x70,
0x70, 0x70, 0xF0, 0xF0, 0xE0, 0xE0, 0xE0, 0xC0, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xC0, 0xE0, 0xF0,
0x78, 0x3C, 0x1E, 0x0F, 0x07, 0x03, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x03, 0x07, 0xFF, 0xFF,
0xFE, 0xF8, 0x00, 0x00, 0x00, 0x78, 0x7C, 0x7F, 0x7F, 0x77, 0x71, 0x70, 0x70, 0x70, 0x70, 0x70,
0x70, 0x70, 0x70, 0xFF, 0xFF, 0xFF, 0xFF, 0x70, 0x70, 0x70, 0x70, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x83, 0xFF, 0xFF,
0xFF, 0xFE, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x38, 0x38, 0x38, 0x38, 0x38, 0x38, 0x38,
0x3F, 0x3F, 0x3F, 0x3F, 0x38, 0x38, 0x38, 0x38, 0x38, 0x38, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x38, 0x3C, 0x3E, 0x3F, 0x3F, 0x3B, 0x39, 0x38, 0x38, 0x38, 0x38, 0x38, 0x38, 0x38, 0x38, 0x38,
0x38, 0x38, 0x38, 0x00, 0x00, 0x00, 0x00, 0x0E, 0x1E, 0x1C, 0x1C, 0x3C, 0x38, 0x38, 0x38, 0x38,
0x38, 0x38, 0x38, 0x1C, 0x1E, 0x1F, 0x0F, 0x07, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x3F, 0x3F, 0x3F, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0E, 0x1E, 0x1C, 0x1C, 0x38, 0x38, 0x38, 0x38, 0x38,
0x38, 0x38, 0x3C, 0x1C, 0x1E, 0x0F, 0x0F, 0x07, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x80, 0xE0, 0xF0, 0xF8, 0x7C, 0x3E, 0x1E, 0x0E, 0x0F, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07,
0x0F, 0x0E, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07,
0x07, 0x07, 0x07, 0x07, 0x07, 0xE7, 0xFF, 0xFF, 0xFF, 0x3F, 0x0F, 0x00, 0x00, 0x00, 0x00, 0x00,
0xE0, 0xF8, 0xFC, 0xFE, 0x1E, 0x0F, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x0F, 0x1E, 0xFE, 0xFC,
0xF8, 0xF0, 0x00, 0x00, 0x00, 0x00, 0x00, 0xC0, 0xF0, 0xF8, 0xFC, 0x3E, 0x1E, 0x0F, 0x07, 0x07,
0x07, 0x07, 0x07, 0x0F, 0x0F, 0x1E, 0xFE, 0xFC, 0xF8, 0xE0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0xC0, 0xF0, 0xF8, 0xFC, 0x3E, 0x0E, 0x0F, 0x07, 0x07, 0x07, 0x07, 0x0F, 0x0F, 0x3E, 0xFE, 0xFC,
0xF8, 0xE0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xF8, 0xFF, 0xFF, 0xFF, 0xC3, 0xE0, 0xE0, 0x70,
0x70, 0x70, 0x70, 0x70, 0x70, 0xF0, 0xF0, 0xE0, 0xE0, 0xC0, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xC0, 0xF0, 0xFC, 0xFF, 0x7F, 0x1F, 0x07,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x0F, 0x1F, 0x1F, 0xBE, 0xFC, 0xF8, 0xF0,
0xF0, 0xE0, 0xF0, 0xF8, 0xF8, 0xBE, 0x1F, 0x0F, 0x07, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1F,
0x7F, 0xFF, 0xFF, 0xF0, 0xC0, 0xC0, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0xC0, 0xC0, 0xE0, 0xFF,
0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0x01, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00,
0x3F, 0xFF, 0xFF, 0xFF, 0xC1, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x83,
0xFF, 0xFF, 0xFF, 0xFE, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xE0, 0xF8,
0xFE, 0xFF, 0x3F, 0x0F, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xF0,
0xFC, 0xFE, 0xFE, 0x0F, 0x07, 0x03, 0x01, 0x01, 0x00, 0x01, 0x01, 0x03, 0x03, 0x07, 0x0F, 0xFF,
0xFE, 0xFC, 0xF0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x03, 0x03, 0x03, 0x03,
0x03, 0x03, 0x03, 0x03, 0x01, 0x81, 0xF0, 0xFF, 0xFF, 0x7F, 0x07, 0x00, 0x00, 0x00, 0x00, 0x3F,
0xFF, 0xFF, 0xFF, 0xE0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xE0, 0xFF,
0xFF, 0xFF, 0x1F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x07, 0x0F, 0x1F, 0x1E, 0x3C, 0x38,
0x38, 0x38, 0x38, 0x38, 0x38, 0x1C, 0x1E, 0x1F, 0x0F, 0x07, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x30, 0x3E, 0x3F, 0x3F, 0x1F, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x07, 0x0F, 0x1F, 0x1E, 0x1C, 0x3C, 0x38, 0x38,
0x38, 0x38, 0x38, 0x38, 0x3C, 0x1C, 0x1E, 0x1F, 0x0F, 0x07, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
0x1C, 0x1C, 0x3C, 0x38, 0x38, 0x38, 0x38, 0x38, 0x38, 0x3C, 0x1C, 0x1E, 0x1F, 0x0F, 0x07, 0x03,
0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x07, 0x0F, 0x1F, 0x1F, 0x3C, 0x3C, 0x38,
0x38, 0x38, 0x38, 0x3C, 0x1C, 0x1F, 0x0F, 0x07, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0xE0, 0xE0, 0xE0, 0xFC, 0xFC, 0xFC, 0xE0, 0xE0, 0xE0, 0x00, 0xE0, 0xE0, 0xE0, 0xE0, 0xE0,
0xE0, 0xE0, 0xE0, 0xE0, 0x00, 0xFF, 0xFF, 0xFF, 0x1F, 0x7E, 0xF8, 0xC0, 0x00, 0x00, 0x00, 0x00,
0xC0, 0xF8, 0x7E, 0x1F, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1C,
0x3E, 0x3E, 0x3E, 0x1C, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFC, 0xFC, 0x1C, 0xEC, 0xEC, 0xEC, 0x1C,
0xFC, 0xBC, 0x0C, 0xAC, 0xAC, 0x0C, 0xAC, 0xEC, 0xFC, 0x00, 0x00, 0xFC, 0x04, 0xE4, 0x14, 0x14,
0x14, 0xE4, 0x04, 0x04, 0xC4, 0x44, 0x44, 0xC4, 0x04, 0x04, 0xFC, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x7F, 0x03, 0x0C, 0x70, 0x70, 0x0C, 0x03, 0x7F, 0x00, 0x74, 0x54, 0x54, 0x7C, 0x00, 0x00,
0x7C, 0x04, 0x04, 0x7C, 0x00, 0x00, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x07, 0x07, 0x07, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x7F, 0x7F, 0x7F,
0x00, 0x00, 0x03, 0x0F, 0x3F, 0x78, 0x78, 0x3F, 0x0F, 0x03, 0x00, 0x00, 0x7F, 0x7F, 0x7F, 0x00,
0x70, 0x70, 0x70, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x1F, 0x1F, 0x1C, 0x1B, 0x1B, 0x1B, 0x1C, 0x1F, 0x1F, 0x18, 0x1F, 0x1F, 0x18, 0x1F, 0x1F,
0x1F, 0x00, 0x00, 0x1F, 0x10, 0x13, 0x14, 0x14, 0x14, 0x13, 0x10, 0x10, 0x17, 0x10, 0x10, 0x17,
0x10, 0x10, 0x1F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x70, 0x1E, 0x13, 0x13, 0x1E, 0x70, 0x00,
0x7C, 0x40, 0x40, 0x7C, 0x00, 0x04, 0x7E, 0x44, 0x00, 0x00, 0x7C, 0x44, 0x44, 0x7C, 0x00, 0x00
};



void i2c_start()  // high low flanke von SDA wenn SCL high
{
  SCL_LOW;
  SCL_SET_WRITE;
  SDA_HIGH;
  SDA_SET_WRITE;
  _delay_us(HALF_CLK);
  SCL_HIGH;
  _delay_us(HALF_CLK);
  SDA_LOW;
  _delay_us(HALF_CLK);
}

void i2c_stop() // low high flanke von SDA wenn SCL high
{
  //SCL_LOW;
  SDA_LOW;
  SDA_SET_WRITE;
  SCL_HIGH;
  _delay_us(HALF_CLK);
  SDA_HIGH;
  _delay_us(HALF_CLK);
}

/*
unsigned char i2c_read(unsigned char ack)
{
unsigned char i;
unsigned char tmp = 0;
SCL_LOW;
_delay_us(HALF_CLK);
SDA_HIGH;
SDA_SET_READ;
_delay_us(HALF_CLK);
for (i=0; i<8; i++)
{
SCL_HIGH;
_delay_us(FULL_CLK);
tmp=tmp<<1;
if (SDA_READ > 0) tmp=tmp+1;
SCL_LOW;
_delay_us(FULL_CLK);
}
// ACK write
if (ack > 0) SDA_LOW;
else SDA_HIGH;
SDA_SET_WRITE;
_delay_us(HALF_CLK);
SCL_HIGH;
_delay_us(FULL_CLK);
SCL_LOW;
_delay_us(FULL_CLK);
return tmp;
}
*/


unsigned char i2c_write(unsigned char in)
{
unsigned char i;
unsigned char tmp;
SCL_LOW;
_delay_us(HALF_CLK);
SDA_LOW;
SDA_SET_WRITE;
_delay_us(HALF_CLK);
for (i=0; i<8; i++)
{
if ((in & 0x80) > 0) SDA_HIGH;
else SDA_LOW;
_delay_us(HALF_CLK);
SCL_HIGH;
_delay_us(FULL_CLK);
SCL_LOW;
_delay_us(HALF_CLK);
in<<=1;
}
// ACK read
SDA_HIGH;
SDA_SET_READ;
_delay_us(HALF_CLK);
SCL_HIGH;
_delay_us(FULL_CLK);
tmp = SDA_READ;
SCL_LOW;
_delay_us(FULL_CLK);
return tmp;
}


void paint(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2, uint8_t xw, uint8_t yw, uint8_t onoff)
{
uint16_t ptr=0;
y1 = y1 / 8;
y2 = y2 / 8;
yw = yw / 8;
i2c_start();
i2c_write(0x78);
i2c_write(OLED_CONTROL_BYTE_CMD_STREAM);
i2c_write(OLED_CMD_SET_COLUMN_RANGE);     i2c_write(x2); i2c_write(x2+xw-1);
i2c_write(OLED_CMD_SET_PAGE_RANGE);       i2c_write(y2); i2c_write(y2+yw-1);
i2c_stop();

ptr=x1*y1;
i2c_start();
i2c_write(0x78);
i2c_write(OLED_CONTROL_BYTE_DATA_STREAM);

if (onoff == 1)
{
for (uint8_t yi=0; yi<yw; yi++) {
ptr = (y1+yi)*120;
for (uint8_t xi=0; xi<xw; xi++) { i2c_write(pgm_read_byte(&font[ptr + xi + x1])); }
}
}
else
{
for (uint8_t yi=0; yi<yw; yi++) {
ptr = (y1+yi)*120;
for (uint8_t xi=0; xi<xw; xi++) { i2c_write(0); }
}
}

i2c_stop();

}


void display_init()
{
  i2c_start();
  i2c_write(0x78);
  i2c_write(OLED_CONTROL_BYTE_CMD_STREAM);
  //i2c_write(OLED_CMD_SET_MUX_RATIO);      i2c_write(0x3f);
  //i2c_write(OLED_CMD_SET_DISPLAY_OFFSET); i2c_write(0x00);
  //i2c_write(OLED_CMD_SET_DISPLAY_START_LINE);
  i2c_write(OLED_CMD_SET_SEGMENT_MODE_REMAP);
  i2c_write(OLED_CMD_SET_COM_SCAN_MODE_REMAPPED);
  //i2c_write(0xa0);
  //i2c_write(0xc8);
  //i2c_write(OLED_CMD_SET_COM_PIN_MAP);     i2c_write(0x01);
  //i2c_write(OLED_CMD_SET_CONTRAST);        i2c_write(0x7f);
  //i2c_write(OLED_CMD_DISPLAY_RAM);
  //i2c_write(OLED_CMD_DISPLAY_NORMAL);
  i2c_write(OLED_CMD_SET_DISPLAY_CLK_DIV); i2c_write(0x80);
  i2c_write(OLED_CMD_SET_CHARGE_PUMP);     i2c_write(0x14);
  i2c_write(OLED_CMD_DISPLAY_ON);
  i2c_write(OLED_CMD_SET_MEMORY_ADDR_MODE); i2c_write(0x00);
  i2c_stop();
}


void display_clear()
{
i2c_start();
i2c_write(0x78);
i2c_write(OLED_CONTROL_BYTE_CMD_STREAM);
i2c_write(OLED_CMD_SET_COLUMN_RANGE);     i2c_write(0x00); i2c_write(0x7f);
i2c_write(OLED_CMD_SET_PAGE_RANGE);       i2c_write(0x00); i2c_write(0x07);
i2c_stop();

uint16_t ptr=0;
for(uint8_t i1=0;i1<64;i1++){
  i2c_start();
  i2c_write(0x78);
  i2c_write(OLED_CONTROL_BYTE_DATA_STREAM);
  for (uint8_t i2=0; i2<16; i2++) { 
    i2c_write(0);
    ptr++; 
  }
  i2c_stop();   
}
}


void display_view_init()
{
paint(47,64, 70,32, 5, 8, 1); // dezimalpunkt
paint( 1,64,  0, 0,10,16, 1);
paint(11,64,  0,48,10,16, 1);
paint(21,64,105, 0,22,16, 1);
}

// mode  == 0: manuell
// onoff == 0: off
void display_mode(uint8_t mode, uint8_t onoff)
{
if ( mode > 0) { paint(97,72,105,48,22, 8, 1); } else { paint(97,64,105,48,22, 8, 1); }
if (onoff > 0) { paint(75,64,108,24,17,16, 1); } else { paint(57,64,108,24,17,16, 1); }
}


void display_value(uint8_t val1, uint8_t val2, uint8_t val3)
{
uint8_t offsetx[10] = {94,0,24,47,70,94, 0,24,47,70};
uint8_t offsety[10] = {32,0, 0, 0, 0, 0,32,32,32,32};

paint(offsetx[val1],offsety[val1],20,8,22,32, val1);
paint(offsetx[val2],offsety[val2],45,8,22,32, 1);
paint(offsetx[val3],offsety[val3],80,8,22,32, 1);
}


void display_progress(unsigned char val) {
uint8_t width = 75;
val=(uint16_t)(val)*width/100;
if (val>width) val=width;
i2c_start();
i2c_write(0x78);
i2c_write(OLED_CONTROL_BYTE_CMD_STREAM);
i2c_write(OLED_CMD_SET_COLUMN_RANGE);     i2c_write(20);   i2c_write(20 + width + 1);
i2c_write(OLED_CMD_SET_PAGE_RANGE);       i2c_write(0x06); i2c_write(0x07);
i2c_stop();

i2c_start();
i2c_write(0x78);
i2c_write(OLED_CONTROL_BYTE_DATA_STREAM);
i2c_write(255);
for (uint8_t i=0;   i<val;   i++) i2c_write(255);
for (uint8_t i=val; i<width; i++) i2c_write(129);
i2c_write(255);
i2c_stop();
}

