//#define F_CPU 8000000UL  // vor #include <util/delay.h>
#include <avr/io.h>           
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include "display.h"

// Hinweise ATMEGA16 und STK500: 
// Chip in SCKT3100A3 + Brcke ISP6PIN - SPROG3

// Hinweise Atiny45 und STK500:  
// Chip in SCKT3400D1 + Brcke ISP6PIN - SPROG1
// RST on PORTE to PB5 on PORTB.
// XT1 on PORTE to PB3 on PORTB.
// wichtig: RSTDISBL in den Fuses setzen, sonst funktioniert PB5 nicht richtig, eventuell PB5 nicht nutzen
// Fuses HIGH,LOW: 0xDF,0xE2


#define BUTTON_INC_READ   ((PINB & (1<<5)) == 0)
#define BUTTON_DEC_READ   ((PINB & (1<<3)) == 0)
#define RELAY_ON          (PORTB |=  (1<<2))
#define RELAY_OFF         (PORTB &= ~(1<<2))
#define RELAY_INIT        (DDRB  |=  (1<<2))
//#define RELAY_ON          asm volatile ("nop")
//#define RELAY_OFF         asm volatile ("nop")
#define ADC_MUX            (1<<MUX1)            // Referenz auf Vcc; ADC2 = PB4
#define ADC_THRESHOLD      22                   // 50 Watt > 112 mV Spitze-Spitze > 22 Bins

static volatile uint16_t adc_h, adc_l, timer_cnt;
uint8_t eeFooByte EEMEM = 0;


void adc_task()
{
uint16_t tmp_adc = 0;
uint16_t tmp_l = adc_l;
uint16_t tmp_h = adc_h;
ADCSRA |= (1<<ADSC);         
while ( ADCSRA & (1<<ADSC) )
{}
tmp_adc = ADCW;
if (tmp_h > tmp_l+100) tmp_h = tmp_h - 10;
if (tmp_l+100 < tmp_h) tmp_l = tmp_l + 10;
if (tmp_h > tmp_l) tmp_h = tmp_h - 1;
if (tmp_l < tmp_h) tmp_l = tmp_l + 1;
if (tmp_adc > tmp_h) tmp_h = tmp_adc;
if (tmp_adc < tmp_l) tmp_l = tmp_adc;
adc_l = tmp_l;
adc_h = tmp_h;
}


void on_off_task(uint16_t val)
{
int val_percent;
int val_percent_old = 0;

// val/10 sekunden ein
timer_cnt=0;
while(1)
{
val_percent = timer_cnt*10/val;
if (val_percent != val_percent_old) { display_progress(val_percent); }
if (val_percent > 100) { break; }
val_percent_old = val_percent;
_delay_ms(1);
}

// 5 sekunden aus
RELAY_OFF;
display_mode(1,0);
timer_cnt=0;
while(1)
{
val_percent = 100 - timer_cnt*10/50;
if (val_percent != val_percent_old) { display_progress(val_percent); }
if (val_percent <= 0) { break; }
val_percent_old = val_percent;
_delay_ms(1);
}
RELAY_ON; 
display_mode(1,1);

// 1 sekunde schutzzeit
_delay_ms(1000);
}


// genau 10ms Takt im Mittel, 8 MHz/1024/78 = 100 Hz
ISR(TIMER0_OVF_vect) {
TCNT0 = -78;
timer_cnt++;
adc_task();
}

 
int main (void) {

// ADC init ATMEGA16
//ADCSRA = (1<<ADEN) | (1<<ADPS1) | (1<<ADPS2); // Frequenzvorteiler und Einschalten
//ADMUX = (1<<REFS0); // Referenz auf Vcc; ADC0, ADC0 = PA0

// ADC init Atiny45
ADCSRA = (1<<ADEN) | (1<<ADPS1) | (1<<ADPS2); // Frequenzvorteiler und Einschalten
ADMUX = ADC_MUX;

// Timer init ATMEGA16
//TCCR0 = (1 << CS02) | (0 << CS01)| (1 << CS00); // prescaler 1024
//TIMSK |= (1<<TOIE0);// Overflow Interrupt erlauben

// Timer init Atiny45
TCCR0B = (1 << CS02) | (0 << CS01)| (1 << CS00); // prescaler 1024
TIMSK |= (1<<TOIE0);// Overflow Interrupt erlauben

uint8_t button_1_old = 0;
uint8_t button_2_old = 0;
uint8_t button_3_old = 0;
uint8_t val = 50;
uint8_t oldval = 0;
uint8_t mode = 1;
uint8_t onoff = 1;
uint8_t cnt1 = 0;
uint8_t cnt2 = 0;
uint16_t adc_val;
uint8_t eeprom_cnt = 0;
adc_h = 0;
adc_l = 10000;

display_init();
display_clear();
display_view_init();
display_progress(0);
display_mode(mode, onoff);

adc_task();

val = eeprom_read_byte (&eeFooByte);
if (val > 100) val = 100;
if (val <  10) val = 10;

sei(); //enable interrupt

RELAY_INIT;
RELAY_ON;

while(1)
{

uint8_t button = 0;

button = BUTTON_INC_READ && BUTTON_DEC_READ;
if ((button > 0) && (button_3_old == 0)) { 
  if (mode > 0 ) { mode = 0; onoff = 0; } 
  else if ((mode == 0) && (onoff == 0)) { onoff = 1; } 
  else { mode = 1; }
  display_mode(mode, onoff);
  if (onoff == 0) RELAY_OFF; else RELAY_ON; 
}
button_3_old = button;

if (button == 0) {
button = BUTTON_INC_READ;
if ((button  > 0) && (button_1_old == 0) && (val < 100)) val = val + 1;
if ((button  > 0) && (button_1_old > 0)) cnt1++; else cnt1 = 0;
if ((cnt1 >= 10) && (val < 100))  val = val + 1;
button_1_old = button;
}

if (button == 0) {
button = BUTTON_DEC_READ;
if ((button  > 0) && (button_2_old == 0) && (val > 10)) val = val - 1;
if ((button  > 0) && (button_2_old > 0)) cnt2++; else cnt2 = 0;
if ((cnt2 >= 10) && (val > 10))  val = val - 1;
button_2_old = button;
}

if (oldval != val) display_value(val/100%10, val/10%10, val%10);	
oldval = val;

adc_val = (adc_h - adc_l);
if ((mode > 0) && (adc_val > ADC_THRESHOLD)) {
  on_off_task(val); } 

if (eeprom_cnt >= 100) {
eeprom_cnt = 0;
eeprom_update_byte(&eeFooByte, val); }
eeprom_cnt++;

_delay_ms(50);
}
                        
return 0;                 
}
